
#include "complexes.h"
#include <math.h>
#include <stdio.h>

float abs (Complexe c)
{
	return sqrt(c.x*c.x + c.y*c.y);
}

Complexe operator- (Complexe c)
{
	Complexe r;
	r.x = -c.x;
	r.y = -c.y;
	return r;
}

bool operator== (Complexe a, Complexe b)
{
	return a.x == b.x && a.y == b.y;
}

bool operator == (Complexe a, float x)
{
	return a.x == x && a.y == 0;
}

bool operator == (float x, Complexe a)
{
	return a.x == x && a.y == 0;
}

Complexe I ()
{
	Complexe c;
	c.x = 0;
	c.y = 1;
	return c;
}
/*
bool operator == (Complexe c)
{
	return x == c.x && y == c.y;
}

bool operator == (float x)
{
	return x == c.x && y == 0;
}
*/
Complexe r (float f)
{
	Complexe c;
	c.x = f;
	c.y = 0;
	return c;
}

void Complexe::disp ()
{
	printf("%f + %f*I\n", x, y);
}

Complexe Complexe::DtoH2 ()
{
	return (I()*(*this) - 1.)/(I() - (*this));
}

Complexe Complexe::H2toD ()
{
	return (I()*(*this) + 1)/((*this) + I());
}

Complexe Complexe::isom_H2 (float a, float b, float c, float d)
{
	Complexe res;
	res = ((*this)*a + b)/((*this)*c + d);
	return res;
}

Complexe Complexe::isom_disque (float a, float b, float c, float d)
{
	float det = a*d - b*c;
	Complexe a2, b2, c2, d2;
	if (det < 0)
	{
		a2 = (I()*r(-b-c) + r(a-d))/2;
		b2 = (I()*r(-d-a) + r(b-c))/2;
		c2 = (I()*r(-a-d) + r(c-b))/2;
		d2 = (I()*r(-b-c) + r(d-a))/2;
	}else
	{
		a2 = (I()*r(c-b) - r(d+a))/2;
		b2 = (I()*r(d-a) - r(b+c))/2;
		c2 = (I()*r(a-d) - r(b+c))/2;
		d2 = (I()*r(b-c) - r(d+a))/2;
	}
	Complexe res;
	if (det < 0)
	{
		//printf("conjugaison !\n");
		y = -y; //conjugaison
	}
	res = homo(a2, b2, c2, d2);
	if (det < 0)
	{
		y = -y; //remet comme avant
	}
	return res;
}

//#define EPSILON_homo	.00001 //pour éviter la division par zéro dans ho

Complexe Complexe::homo (Complexe a, Complexe b, Complexe c, Complexe d)
{
	Complexe c2;
	//if ((c*(*this) + d).cnorm() < EPSILON_homo)
	c2 = (a*(*this) + b)/(c*(*this) + d);
	//else
	//	c2 = a/c;
	return c2;
}

void Complexe::conj ()
{
	y = -y;
}

inline float carre (float x)
{
	return x*x;
}

float Complexe::cnorm ()
{
	return carre(x) + carre(y);
}

void Complexe::inverse ()
{
	float cr = cnorm();
	x = x/cr;
	y = -y/cr;
}

Complexe Complexe::operator / (Complexe c)
{
	Complexe c2;
	c2 = c;
	c2.inverse();
	c2 *= (*this);
	return c2;
}

Complexe Complexe::operator / (float f)
{
	Complexe c2;
	c2.x = x/f;
	c2.y = y/f;
	return c2;
}

void Complexe::operator += (Complexe c)
{
	x += c.x;
	y += c.y;
}

Complexe Complexe::operator + (Complexe c)
{
	Complexe c2;
	c2.x = x + c.x;
	c2.y = y + c.y;
	return c2;
}

Complexe Complexe::operator + (float f)
{
	Complexe c2;
	c2.x = x + f;
	c2.y = y;
	return c2;
}

Complexe Complexe::operator - (Complexe c)
{
	Complexe c2;
	c2.x = x - c.x;
	c2.y = y - c.y;
	return c2;
}

Complexe Complexe::operator - (float f)
{
	Complexe c2;
	c2.x = x - f;
	c2.y = y;
	return c2;
}

void Complexe::operator *= (Complexe c)
{
	float rx = x;
	x = x*c.x - y*c.y;
	y = y*c.x + rx*c.y;
}

Complexe Complexe::operator * (Complexe c)
{
	Complexe c2;
	c2.x = x*c.x - y*c.y;
	c2.y = y*c.x + x*c.y;
	return c2;
}

Complexe Complexe::operator * (float f)
{
	Complexe c2;
	c2.x = x*f;
	c2.y = y*f;
	return c2;
}
