#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <pari/pari.h>
#include <SDL/SDL.h>
#include "resize.h"

#define DX	16000 //dimensions de l'image produite
#define DY	16000

#define SX	3 //tailles de la zone calculé
#define SY	3
#define CX	0 //centre
#define CY	1

#define DX2	1000 //dimensions de l'image produite
#define DY2	1000

#define NG	2 //nombre de générateurs
GEN beta;
GEN t[NG];
GEN c; //nombre complexe utilisé dans les calculs
GEN borne;
long prec = 6;
#define NITER	30 //nombre d'itérations
GEN CX_, CY_, SX_, SY_;

#define NITER2	200000000 //nombre d'itérations pour la deuxième méthode

void InitSem ()
{
	c = cgetc(prec);
	beta = cgetc(prec);
	gel(beta, 1) = dbltor(0.593691181920914);
	//gsqrt(stoi(1), prec); //gsqrt(gdiv(stoi(3), stoi(2)), prec); //dbltor(1); //dbltor(.65); //stoi(1);
	gel(beta, 2) = dbltor(1.19615833607094);
	//gsqrt(stoi(3), prec); //gsqrt(gdiv(stoi(3), stoi(2)), prec); //gsqrt(dbltor(2), prec); //dbltor(.3); //stoi(1);
	t[0] = stoi(0);
	t[1] = stoi(1);
	t[2] = stoi(2);
	t[3] = stoi(3);
	SX_ = dbltor(SX);
	SY_ = dbltor(SY);
	CX_ = dbltor(CX + SX/2.);
	CY_ = dbltor(CY + SY/2.);
	
	borne = gdiv(stoi(1), gsub(gabs(beta, prec), stoi(1)));
}

bool decompose (GEN p, int n)
{
	if (n)
	{
		bool res = false;
		p = gmul(p, beta);
		if (gcmp(gabs(p, prec), borne) < 0)
		{
			res = decompose(p, n-1);
			if (res)
				return true;
		}
		{
			p = gsub(p, t[1]);
			if (gcmp(gabs(p, prec), borne) < 0)
			{
				res = decompose(p, n-1);
			}
		}
		return res;
	}
	return true;
}

Uint32 Color (int x, int y)
{
	Uint32 res = 0xffffffff;
	pari_sp av = avma;
	gel(c, 1) = gsub(gdiv(gmul(stoi(x), SX_), stoi(DX)), CX_);
	gel(c, 2) = gsub(gdiv(gmul(stoi(y), SY_), stoi(DY)), CY_);
	if (decompose(c, NITER))
	{
		res = 0x000000ff;
	}
	avma = av;
	return res;
}

void Draw (SDL_Surface *s)
{
	int x,y;
	Uint32 *pix = (Uint32 *)s->pixels;
	for (y=0;y<s->h;y++)
	{
		for (x=0;x<s->w;x++)
		{
			*pix = Color(x, y);
			pix++;
		}
		pix += (s->pitch/4 - s->w);
	}
}

void draw (SDL_Surface *s, GEN c, Uint32 color)
{
	int x = (rtodbl(gel(c, 1)) + CX + SX/2)*DX/SX;
	int y = (rtodbl(gel(c, 2)) + CY + SY/2)*DY/SY;
	if (x >= 0 && x < DX && y >= 0 && y < DY)
	{
		*((Uint32 *)s->pixels + x + y*(s->pitch)/4) = color;
	}
}

void Draw2 (SDL_Surface *s)
{
	pari_sp av = avma;
	gel(c, 1) = dbltor(0);
	gel(c, 2) = dbltor(0);
	
	SDL_FillRect(s, NULL, 0xffffffff);
	
	int i;
	for (i=0;i<NITER2;i++)
	{
		if (i%100 == 0)
		{
			avma = av;
			gel(c, 1) = dbltor(0);
			gel(c, 2) = dbltor(0);
		}
		draw(s, c, 0x000000ff);
		c = gadd(c, t[rand()%NG]);
		c = gdiv(c, beta);
	}
}

int main (int nargs, char **args)
{
	printf("Initialisation de pari...\n");
	pari_init(1000000,2);
	InitSem();
	//outbrute(beta);
	printf("Initialisation de la SDL...\n");
	if (SDL_Init(SDL_INIT_VIDEO))
	{
		fprintf(stderr,"Initialisation de la SDL rate : %s\n",SDL_GetError());
		exit(EXIT_FAILURE);
	}
	
	printf("Creation de la surface...\n");
	SDL_Surface *s = SDL_CreateRGBSurface(SDL_SWSURFACE, DX, DY, 32, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
	if (!s)
	{
		printf("Pas assez de RAM !\n");
		return -1;
	}
	
	printf("Dessin...\n");
	Draw2(s);
	
	printf("Redimensionnement...\n");
	SDL_Surface *s2 = SDL_CreateRGBSurface(SDL_SWSURFACE, DX2, DY2, 32, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
	Copy(s, s2, NULL);
	
	printf("Enregistrement...\n");
	SDL_SaveBMP(s2, "result.bmp");
	
	SDL_Quit();
	printf("Goodbye !\n");
	return 0;
}