#include <SDL/SDL.h>

double min (double a, double b)
{
	if (a < b)
		return a;
	return b;
}

double max (double a, double b)
{
	if (a < b)
		return b;
	return a;
}

struct Rect
{
	double x,y,w,h;
};

double Surf (int x, int y, Rect *r)
{
	double ix = min((double)(x+1), r->x+r->w) - max((double)x, r->x);
	double iy = min((double)(y+1), r->y+r->h) - max((double)y, r->y);
	if (ix < 0 || iy < 0)
		return 0.;
	return ix*iy;
}

Uint32 GetPix (SDL_Surface *s, Rect *re, SDL_PixelFormat *f)
{
	int x,y;
	double p;
	Uint8 r0, v0, b0;
	double r = 0., v = 0., b = 0.;
	double ps = 0.;
	Uint32 *pix = (Uint32 *)s->pixels;
	for (y=(int)re->y; y<(int)(re->y+re->h+1); y++)
	{
		for (x=(int)re->x; x<(int)(re->x+re->w+1); x++)
		{
			//calcule la surface de l'intersection avec re
			p = Surf(x, y, re);
			//détermine la couleur du pixel courant
			if (x >= 0 && y >= 0 && x < s->w && y < s->h)
				SDL_GetRGB(*(pix + x + y*(s->pitch/4)), s->format, &r0, &v0, &b0);
			else
			{
				r0 = 0;
				v0 = 0;
				b0 = 0;
			}
			//fait la somme pondérée
			r += p*r0;
			v += p*v0;
			b += p*b0;
			ps += p;
		}
	}
	
	//vérifie qu'il n'y a pas d'erreur
	if (abs(ps - re->w*re->h) > .00000001)
	{
		printf("\n!!! %lf != %lf !!!!\n\n", ps, re->w*re->h);
	}
	
	//divise par la surface totale
	r /= (re->w*re->h);
	v /= (re->w*re->h);
	b /= (re->w*re->h);
	//convertit
	return SDL_MapRGB(f, (int)(.5+r), (int)(.5+v), (int)(.5+b));
}

void Copy (SDL_Surface *f, SDL_Surface *d, SDL_Rect *r_) //copie l'image f dans le rectangle r sur l'image d (en retaillant)
{
	SDL_Rect r;
	if (r_)
		r = *r_;
	else
	{
		r.x = 0;
		r.y = 0;
		r.w = d->w;
		r.h = d->h;
	}
	int x,y;
	Uint32 *pix = (Uint32 *)d->pixels;
	Rect r2;
	for (y=r.y;y<r.y+r.h;y++)
	{
		for (x=r.x;x<r.x+r.w;x++)
		{
			r2.x = (double)(x - r.x)*f->w/r.w;
			r2.y = (double)(y - r.y)*f->h/r.h;
			r2.w = (double)f->w/r.w;
			r2.h = (double)f->h/r.h;
			if (x >= 0 && y >= 0 && x < d->w && y < d->h)
				*(pix + x + (d->pitch/4)*y) = GetPix(f, &r2, d->format);
		}
		//SDL_BlitSurface(d, NULL, ecran, NULL);
		//SDL_Flip(ecran);
	}
}